@@ -4,9 +4,9 @@
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
  * You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -134,7 +134,8 @@ public class DecompilerProvider extends NavigatableComponentProviderAdapter
 
 		decompilerOptions = new DecompileOptions();
 		initializeDecompilerOptions();
-		controller = new DecompilerController(this, decompilerOptions, clipboardProvider);
+		controller =
+			new DecompilerController(getTool(), this, decompilerOptions, clipboardProvider);
 		DecompilerPanel decompilerPanel = controller.getDecompilerPanel();
 
 		// TODO move the hl controller into the panel
@@ -165,6 +166,7 @@ public class DecompilerProvider extends NavigatableComponentProviderAdapter
 
 		plugin.getTool().addServiceListener(serviceListener);
 		programListener = new DecompilerProgramListener(controller, redecompileUpdater);
+		setDefaultFocusComponent(controller.getDecompilerPanel());
 	}
 
 //==================================================================================================
@@ -250,7 +252,6 @@ public class DecompilerProvider extends NavigatableComponentProviderAdapter
 
 	@Override
 	public boolean goTo(Program gotoProgram, ProgramLocation location) {
-
 		if (!isConnected()) {
 			if (program == null) {
 				// Special Case: this 'disconnected' provider is waiting to be initialized
@@ -287,24 +288,19 @@ public class DecompilerProvider extends NavigatableComponentProviderAdapter
 		pendingViewerPosition = decompMemento.getViewerPosition();
 	}
 
-	@Override
-	public void requestFocus() {
-		controller.getDecompilerPanel().requestFocus();
-		tool.toFront(this);
-	}
-
 //==================================================================================================
 // DecompilerHighlightService interface methods
 //==================================================================================================
 
 	@Override
-	public DecompilerHighlighter createHighlighter(CTokenHighlightMatcher tm) {
-		return getDecompilerPanel().createHighlighter(tm);
+	public DecompilerHighlighter createHighlighter(Function f, CTokenHighlightMatcher tm) {
+		return getDecompilerPanel().createHighlighter(f, tm);
 	}
 
 	@Override
-	public DecompilerHighlighter createHighlighter(String id, CTokenHighlightMatcher tm) {
-		return getDecompilerPanel().createHighlighter(id, tm);
+	public DecompilerHighlighter createHighlighter(String id, Function f,
+			CTokenHighlightMatcher tm) {
+		return getDecompilerPanel().createHighlighter(id, f, tm);
 	}
 
 //==================================================================================================
@@ -905,6 +901,9 @@ public class DecompilerProvider extends NavigatableComponentProviderAdapter
 		OverridePrototypeAction overrideSigAction = new OverridePrototypeAction();
 		setGroupInfo(overrideSigAction, functionGroup, subGroupPosition++);
 
+		EditPrototypeOverrideAction editOverrideSigAction = new EditPrototypeOverrideAction();
+		setGroupInfo(editOverrideSigAction, functionGroup, subGroupPosition++);
+
 		DeletePrototypeOverrideAction deleteSigAction = new DeletePrototypeOverrideAction();
 		setGroupInfo(deleteSigAction, functionGroup, subGroupPosition++);
 
@@ -961,6 +960,10 @@ public class DecompilerProvider extends NavigatableComponentProviderAdapter
 		EditDataTypeAction editDataTypeAction = new EditDataTypeAction();
 		setGroupInfo(editDataTypeAction, variableGroup, subGroupPosition++);
 
+		// shows the quick editor dialog
+		EditFieldAction editFieldAction = new EditFieldAction();
+		setGroupInfo(editFieldAction, variableGroup, subGroupPosition++);
+
 		//
 		// Listing action for Creating Structure on a Variable
 		//
@@ -1018,6 +1021,13 @@ public class DecompilerProvider extends NavigatableComponentProviderAdapter
 			new RemoveAllSecondaryHighlightsAction();
 		setGroupInfo(removeAllSecondadryHighlightsAction, highlightGroup, subGroupPosition++);
 
+		PreviousHighlightedTokenAction previousHighlightedTokenAction =
+			new PreviousHighlightedTokenAction();
+		setGroupInfo(previousHighlightedTokenAction, highlightGroup, subGroupPosition++);
+
+		NextHighlightedTokenAction nextHighlightedTokenAction = new NextHighlightedTokenAction();
+		setGroupInfo(nextHighlightedTokenAction, highlightGroup, subGroupPosition++);
+
 		String convertGroup = "7 - Convert Group";
 		subGroupPosition = 0;
 		RemoveEquateAction removeEquateAction = new RemoveEquateAction();
@@ -1057,7 +1067,7 @@ public class DecompilerProvider extends NavigatableComponentProviderAdapter
 		//
 		// Search
 		//
-		String searchGroup = "comment2 - Search Group";
+		String searchGroup = "Comment2 - Search Group";
 		subGroupPosition = 0; // reset for the next group
 
 		FindAction findAction = new FindAction();
@@ -1124,6 +1134,8 @@ public class DecompilerProvider extends NavigatableComponentProviderAdapter
 		addLocalAction(setSecondaryHighlightColorChooserAction);
 		addLocalAction(removeSecondaryHighlightAction);
 		addLocalAction(removeAllSecondadryHighlightsAction);
+		addLocalAction(nextHighlightedTokenAction);
+		addLocalAction(previousHighlightedTokenAction);
 		addLocalAction(convertBinaryAction);
 		addLocalAction(convertDecAction);
 		addLocalAction(convertFloatAction);
@@ -1142,8 +1154,10 @@ public class DecompilerProvider extends NavigatableComponentProviderAdapter
 		addLocalAction(decompilerCreateStructureAction);
 		tool.addAction(listingCreateStructureAction);
 		addLocalAction(editDataTypeAction);
+		addLocalAction(editFieldAction);
 		addLocalAction(specifyCProtoAction);
 		addLocalAction(overrideSigAction);
+		addLocalAction(editOverrideSigAction);
 		addLocalAction(deleteSigAction);
 		addLocalAction(renameFunctionAction);
 		addLocalAction(renameLabelAction);
@@ -1166,7 +1180,14 @@ public class DecompilerProvider extends NavigatableComponentProviderAdapter
 	private void setGroupInfo(DockingAction action, String group, int subGroupPosition) {
 		MenuData popupMenuData = action.getPopupMenuData();
 		popupMenuData.setMenuGroup(group);
-		popupMenuData.setMenuSubGroup(Integer.toString(subGroupPosition));
+
+		// Some groups have numbers reach double-digits.  These will not compare correctly unless
+		// padded.  Ensure all string numbers are at least 2 digits.
+		String numberString = Integer.toString(subGroupPosition);
+		if (numberString.length() == 1) {
+			numberString = '0' + numberString;
+		}
+		popupMenuData.setMenuSubGroup(numberString);
 	}
 
 	private void graphServiceRemoved() {
