@@ -4,9 +4,9 @@
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
  * You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,6 +15,8 @@
  */
 package ghidra.app.plugin.core.symboltree;
 
+import static ghidra.framework.main.DataTreeDialogType.*;
+
 import java.awt.*;
 import java.awt.event.ItemListener;
 import java.util.Arrays;
@@ -269,7 +271,7 @@ class EditExternalLocationPanel extends JPanel {
 	 */
 	private void popupProgramChooser() {
 		DataTreeDialog d =
-			new DataTreeDialog(this.getParent(), "Choose External Program", DataTreeDialog.OPEN);
+			new DataTreeDialog(this.getParent(), "Choose External Program", OPEN);
 		final DataTreeDialog dialog = d;
 		d.addOkActionListener(e -> {
 			DomainFile df = dialog.getDomainFile();
@@ -298,7 +300,7 @@ class EditExternalLocationPanel extends JPanel {
 		if (extOriginalLabelTextField != null) {
 			extOriginalLabelTextField.setText(startingOriginalName);
 		}
-		extAddressInputWidget.setAddressFactory(program.getAddressFactory());
+		extAddressInputWidget.setProgram(program);
 		if (startingLocationAddress != null) {
 			extAddressInputWidget.setAddress(startingLocationAddress);
 		}
