@@ -4,9 +4,9 @@
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
  * You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -15,9 +15,14 @@
  */
 package ghidra.dbg.jdi.manager;
 
-import ghidra.dbg.target.TargetConsole.Channel;
-
 public interface JdiConsoleOutputListener {
+	/**
+	 * For console output notifications, indicates whether it is normal or error output
+	 */
+	public static enum Channel {
+		STDOUT, STDERR;
+	}
+
 	/**
 	 * JDI outputted some text
 	 * 
