@@ -1,13 +1,12 @@
 /* ###
  * IP: GHIDRA
- * REVIEWED: YES
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
  * You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -22,13 +21,6 @@ import ghidra.util.exception.UsrException;
  * Indicates a failure to obtain a required lock.
  */
 public class LockException extends UsrException {
-
-	/**
-     * Construct a new LockException
-     */
-    public LockException() {
-        super("Operation requires exclusive access to object.");
-    }
 	
     /**
      * Construct a new LockException with the given message
