@@ -4,9 +4,9 @@
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
  * You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -17,14 +17,16 @@ package sarif.handlers.result;
 
 import sarif.handlers.SarifResultHandler;
 
-public class SarifToolResultHandler extends SarifResultHandler  {
-	
+public class SarifToolResultHandler extends SarifResultHandler {
+
+	@Override
 	public String getKey() {
 		return "Tool";
 	}
 
+	@Override
 	public String parse() {
 		return run.getTool().getDriver().getName();
 	}
-	
+
 }
