@@ -1,54 +1,66 @@
 ## ###
-#  IP: GHIDRA
-# 
-#  Licensed under the Apache License, Version 2.0 (the "License");
-#  you may not use this file except in compliance with the License.
-#  You may obtain a copy of the License at
-#  
-#       http://www.apache.org/licenses/LICENSE-2.0
-#  
-#  Unless required by applicable law or agreed to in writing, software
-#  distributed under the License is distributed on an "AS IS" BASIS,
-#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-#  See the License for the specific language governing permissions and
-#  limitations under the License.
+# IP: GHIDRA
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#      http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
 ##
-import time
+from dataclasses import dataclass, field
 import threading
+import time
+from typing import Any, Optional, Union
 
 import lldb
 
 from . import commands, util
 
+
 ALL_EVENTS = 0xFFFF
 
+
+@dataclass(frozen=False)
 class HookState(object):
-    __slots__ = ('installed', 'mem_catchpoint')
+    installed = False
 
-    def __init__(self):
+    def __init__(self) -> None:
         self.installed = False
-        self.mem_catchpoint = None
 
 
+@dataclass(frozen=False)
 class ProcessState(object):
-    __slots__ = ('first', 'regions', 'modules', 'threads', 'breaks', 'watches', 'visited')
+    first = True
+    # For things we can detect changes to between stops
+    regions = False
+    modules = False
+    threads = False
+    breaks = False
+    watches = False
+    # For frames and threads that have already been synced since last stop
+    visited: set[Any] = field(default_factory=set)
 
-    def __init__(self):
+    def __init__(self) -> None:
         self.first = True
-        # For things we can detect changes to between stops
         self.regions = False
         self.modules = False
         self.threads = False
         self.breaks = False
         self.watches = False
-        # For frames and threads that have already been synced since last stop
         self.visited = set()
 
-    def record(self, description=None):
+    def record(self, description: Optional[str] = None) -> None:
         first = self.first
         self.first = False
+        trace = commands.STATE.require_trace()
         if description is not None:
-            commands.STATE.trace.snapshot(description)
+            trace.snapshot(description)
         if first:
             commands.put_processes()
             commands.put_environment()
@@ -64,11 +76,24 @@ class ProcessState(object):
             hashable_frame = (thread.GetThreadID(), frame.GetFrameID())
             if first or hashable_frame not in self.visited:
                 banks = frame.GetRegisters()
-                commands.putreg(frame, banks.GetFirstValueByName(commands.DEFAULT_REGISTER_BANK))
-                commands.putmem("$pc", "1", from_tty=False)
-                commands.putmem("$sp", "1", from_tty=False)
+                primary = banks.GetFirstValueByName(
+                    commands.DEFAULT_REGISTER_BANK)
+                if primary.value is None:
+                    primary = banks[0]
+                    if primary is not None:
+                        commands.DEFAULT_REGISTER_BANK = primary.name
+                if primary is not None:
+                    commands.putreg(frame, primary)
+                try:
+                    commands.putmem("$pc", "1", result=None)
+                except BaseException as e:
+                    print(f"Couldn't record page with PC: {e}")
+                try:
+                    commands.putmem("$sp-1", "2", result=None)
+                except BaseException as e:
+                    print(f"Couldn't record page with SP: {e}")
                 self.visited.add(hashable_frame)
-        if first or self.regions or self.threads or self.modules:
+        if first or self.regions or self.modules:
             # Sections, memory syscalls, or stack allocations
             commands.put_regions()
             self.regions = False
@@ -89,74 +114,65 @@ class ProcessState(object):
     def record_exited(self, exit_code):
         proc = util.get_process()
         ipath = commands.PROCESS_PATTERN.format(procnum=proc.GetProcessID())
-        commands.STATE.trace.proxy_object_path(
-            ipath).set_value('_exit_code', exit_code)
-
-
-class BrkState(object):
-    __slots__ = ('break_loc_counts',)
-
-    def __init__(self):
-        self.break_loc_counts = {}
-
-    def update_brkloc_count(self, b, count):
-        self.break_loc_counts[b.GetID()] = count
-
-    def get_brkloc_count(self, b):
-        return self.break_loc_counts.get(b.GetID(), 0)
-
-    def del_brkloc_count(self, b):
-        if b not in self.break_loc_counts:
-            return 0  # TODO: Print a warning?
-        count = self.break_loc_counts[b.GetID()]
-        del self.break_loc_counts[b.GetID()]
-        return count
+        procobj = commands.STATE.trace.proxy_object_path(ipath)
+        procobj.set_value('Exit Code', exit_code)
+        procobj.set_value('State', 'TERMINATED')
 
 
 HOOK_STATE = HookState()
-BRK_STATE = BrkState()
 PROC_STATE = {}
 
-def process_event(self, listener, event):
+
+class QuitSentinel(object):
+    pass
+
+
+QUIT = QuitSentinel()
+
+
+def process_event(self, listener: lldb.SBListener,
+                  event: lldb.SBEvent) -> Union[QuitSentinel, bool]:
     try:
         desc = util.get_description(event)
-        #event_process = lldb.SBProcess_GetProcessFromEvent(event)
+        # print(f"Event: {desc}")
+        target = util.get_target()
+        if not target.IsValid():
+            # LLDB may crash on event.GetBroadcasterClass, otherwise
+            # All the checks below, e.g. SBTarget.EventIsTargetEvent, call this
+            print(f"Ignoring {desc} because target is invalid")
+            return False
         event_process = util.get_process()
-        if event_process not in PROC_STATE:
+        if event_process.IsValid() and event_process.GetProcessID() not in PROC_STATE:
             PROC_STATE[event_process.GetProcessID()] = ProcessState()
             rc = event_process.GetBroadcaster().AddListener(listener, ALL_EVENTS)
-            if rc is False:
+            if not rc:
                 print("add listener for process failed")
 
-        commands.put_state(event_process)
+        # NB: Calling put_state on running leaves an open transaction
+        if not event_process.is_running:
+            commands.put_state(event_process)
         type = event.GetType()
         if lldb.SBTarget.EventIsTargetEvent(event):
-            print('Event:', desc)
             if (type & lldb.SBTarget.eBroadcastBitBreakpointChanged) != 0:
-                print("eBroadcastBitBreakpointChanged")
                 return on_breakpoint_modified(event)
             if (type & lldb.SBTarget.eBroadcastBitWatchpointChanged) != 0:
-                print("eBroadcastBitWatchpointChanged")
                 return on_watchpoint_modified(event)
             if (type & lldb.SBTarget.eBroadcastBitModulesLoaded) != 0:
-                print("eBroadcastBitModulesLoaded")
                 return on_new_objfile(event)
             if (type & lldb.SBTarget.eBroadcastBitModulesUnloaded) != 0:
-                print("eBroadcastBitModulesUnloaded")
                 return on_free_objfile(event)
             if (type & lldb.SBTarget.eBroadcastBitSymbolsLoaded) != 0:
-                print("eBroadcastBitSymbolsLoaded")
                 return True
         if lldb.SBProcess.EventIsProcessEvent(event):
             if (type & lldb.SBProcess.eBroadcastBitStateChanged) != 0:
-                print("eBroadcastBitStateChanged")
                 if not event_process.is_alive:
                     return on_exited(event)
                 if event_process.is_stopped:
                     return on_stop(event)
+                if event_process.is_running:
+                    return on_cont(event)
                 return True
             if (type & lldb.SBProcess.eBroadcastBitInterrupt) != 0:
-                print("eBroadcastBitInterrupt")
                 if event_process.is_stopped:
                     return on_stop(event)
             if (type & lldb.SBProcess.eBroadcastBitSTDOUT) != 0:
@@ -164,138 +180,104 @@ def process_event(self, listener, event):
             if (type & lldb.SBProcess.eBroadcastBitSTDERR) != 0:
                 return True
             if (type & lldb.SBProcess.eBroadcastBitProfileData) != 0:
-                print("eBroadcastBitProfileData")
                 return True
             if (type & lldb.SBProcess.eBroadcastBitStructuredData) != 0:
-                print("eBroadcastBitStructuredData")
                 return True
         # NB: Thread events not currently processes
         if lldb.SBThread.EventIsThreadEvent(event):
-            print('Event:', desc)
             if (type & lldb.SBThread.eBroadcastBitStackChanged) != 0:
-                print("eBroadcastBitStackChanged")
                 return on_frame_selected()
             if (type & lldb.SBThread.eBroadcastBitThreadSuspended) != 0:
-                print("eBroadcastBitThreadSuspended")
                 if event_process.is_stopped:
                     return on_stop(event)
             if (type & lldb.SBThread.eBroadcastBitThreadResumed) != 0:
-                print("eBroadcastBitThreadResumed")
                 return on_cont(event)
             if (type & lldb.SBThread.eBroadcastBitSelectedFrameChanged) != 0:
-                print("eBroadcastBitSelectedFrameChanged")
                 return on_frame_selected()
             if (type & lldb.SBThread.eBroadcastBitThreadSelected) != 0:
-                print("eBroadcastBitThreadSelected")
                 return on_thread_selected()
         if lldb.SBBreakpoint.EventIsBreakpointEvent(event):
-            print('Event:', desc)
-            btype = lldb.SBBreakpoint.GetBreakpointEventTypeFromEvent(event);
-            bpt = lldb.SBBreakpoint.GetBreakpointFromEvent(event);
+            btype = lldb.SBBreakpoint.GetBreakpointEventTypeFromEvent(event)
+            bpt = lldb.SBBreakpoint.GetBreakpointFromEvent(event)
             if btype is lldb.eBreakpointEventTypeAdded:
-                print("eBreakpointEventTypeAdded")
                 return on_breakpoint_created(bpt)
             if btype is lldb.eBreakpointEventTypeAutoContinueChanged:
-                print("elldb.BreakpointEventTypeAutoContinueChanged")
                 return on_breakpoint_modified(bpt)
             if btype is lldb.eBreakpointEventTypeCommandChanged:
-                print("eBreakpointEventTypeCommandChanged")
                 return on_breakpoint_modified(bpt)
             if btype is lldb.eBreakpointEventTypeConditionChanged:
-                print("eBreakpointEventTypeConditionChanged")
                 return on_breakpoint_modified(bpt)
             if btype is lldb.eBreakpointEventTypeDisabled:
-                print("eBreakpointEventTypeDisabled")
                 return on_breakpoint_modified(bpt)
             if btype is lldb.eBreakpointEventTypeEnabled:
-                print("eBreakpointEventTypeEnabled")
                 return on_breakpoint_modified(bpt)
             if btype is lldb.eBreakpointEventTypeIgnoreChanged:
-                print("eBreakpointEventTypeIgnoreChanged")
                 return True
             if btype is lldb.eBreakpointEventTypeInvalidType:
-                print("eBreakpointEventTypeInvalidType")
                 return True
             if btype is lldb.eBreakpointEventTypeLocationsAdded:
-                print("eBreakpointEventTypeLocationsAdded")
                 return on_breakpoint_modified(bpt)
             if btype is lldb.eBreakpointEventTypeLocationsRemoved:
-                print("eBreakpointEventTypeLocationsRemoved")
                 return on_breakpoint_modified(bpt)
             if btype is lldb.eBreakpointEventTypeLocationsResolved:
-                print("eBreakpointEventTypeLocationsResolved")
                 return on_breakpoint_modified(bpt)
             if btype is lldb.eBreakpointEventTypeRemoved:
-                print("eBreakpointEventTypeRemoved")
                 return on_breakpoint_deleted(bpt)
             if btype is lldb.eBreakpointEventTypeThreadChanged:
-                print("eBreakpointEventTypeThreadChanged")
                 return on_breakpoint_modified(bpt)
             print("UNKNOWN BREAKPOINT EVENT")
             return True
         if lldb.SBWatchpoint.EventIsWatchpointEvent(event):
-            print('Event:', desc)
-            btype = lldb.SBWatchpoint.GetWatchpointEventTypeFromEvent(event);
-            bpt = lldb.SBWatchpoint.GetWatchpointFromEvent(eventt);
+            btype = lldb.SBWatchpoint.GetWatchpointEventTypeFromEvent(event)
+            bpt = lldb.SBWatchpoint.GetWatchpointFromEvent(event)
             if btype is lldb.eWatchpointEventTypeAdded:
-                print("eWatchpointEventTypeAdded")
-                return on_watchpoint_added(bpt)
+                return on_watchpoint_created(bpt)
             if btype is lldb.eWatchpointEventTypeCommandChanged:
-                print("eWatchpointEventTypeCommandChanged")
                 return on_watchpoint_modified(bpt)
             if btype is lldb.eWatchpointEventTypeConditionChanged:
-                print("eWatchpointEventTypeConditionChanged")
                 return on_watchpoint_modified(bpt)
             if btype is lldb.eWatchpointEventTypeDisabled:
-                print("eWatchpointEventTypeDisabled")
                 return on_watchpoint_modified(bpt)
             if btype is lldb.eWatchpointEventTypeEnabled:
-                print("eWatchpointEventTypeEnabled")
                 return on_watchpoint_modified(bpt)
             if btype is lldb.eWatchpointEventTypeIgnoreChanged:
-                print("eWatchpointEventTypeIgnoreChanged")
                 return True
             if btype is lldb.eWatchpointEventTypeInvalidType:
-                print("eWatchpointEventTypeInvalidType")
                 return True
             if btype is lldb.eWatchpointEventTypeRemoved:
-                print("eWatchpointEventTypeRemoved")
                 return on_watchpoint_deleted(bpt)
             if btype is lldb.eWatchpointEventTypeThreadChanged:
-                print("eWatchpointEventTypeThreadChanged")
                 return on_watchpoint_modified(bpt)
             if btype is lldb.eWatchpointEventTypeTypeChanged:
-                print("eWatchpointEventTypeTypeChanged")
                 return on_watchpoint_modified(bpt)
             print("UNKNOWN WATCHPOINT EVENT")
             return True
         if lldb.SBCommandInterpreter.EventIsCommandInterpreterEvent(event):
-            print('Event:', desc)
             if (type & lldb.SBCommandInterpreter.eBroadcastBitAsynchronousErrorData) != 0:
-                print("eBroadcastBitAsynchronousErrorData")
                 return True
             if (type & lldb.SBCommandInterpreter.eBroadcastBitAsynchronousOutputData) != 0:
-                print("eBroadcastBitAsynchronousOutputData")
                 return True
             if (type & lldb.SBCommandInterpreter.eBroadcastBitQuitCommandReceived) != 0:
-                print("eBroadcastBitQuitCommandReceived")
+                # DO NOT return QUIT here.
+                # For some reason, this event comes just after launch.
+                # Maybe need to figure out *which* interpreter?
                 return True
             if (type & lldb.SBCommandInterpreter.eBroadcastBitResetPrompt) != 0:
-                print("eBroadcastBitResetPrompt")
                 return True
             if (type & lldb.SBCommandInterpreter.eBroadcastBitThreadShouldExit) != 0:
-                print("eBroadcastBitThreadShouldExit")
                 return True
         print("UNKNOWN EVENT")
         return True
-    except RuntimeError as e:
+    except BaseException as e:
         print(e)
-    
+        return False
+
+
 class EventThread(threading.Thread):
     func = process_event
     event = lldb.SBEvent()
-   
-    def run(self):        
+
+    def run(self) -> None:
         # Let's only try at most 4 times to retrieve any kind of event.
         # After that, the thread exits.
         listener = lldb.SBListener('eventlistener')
@@ -303,54 +285,63 @@ class EventThread(threading.Thread):
         target = util.get_target()
         proc = util.get_process()
         rc = cli.GetBroadcaster().AddListener(listener, ALL_EVENTS)
-        if rc is False:
+        if not rc:
             print("add listener for cli failed")
-            return
+            # return
         rc = target.GetBroadcaster().AddListener(listener, ALL_EVENTS)
-        if rc is False:
+        if not rc:
             print("add listener for target failed")
-            return
+            # return
         rc = proc.GetBroadcaster().AddListener(listener, ALL_EVENTS)
-        if rc is False:
+        if not rc:
             print("add listener for process failed")
-            return
-        
+            # return
+
         # Not sure what effect this logic has
         rc = cli.GetBroadcaster().AddInitialEventsToListener(listener, ALL_EVENTS)
-        if rc is False:
-            print("add listener for cli failed")
-            return
+        if not rc:
+            print("add initial events for cli failed")
+            # return
         rc = target.GetBroadcaster().AddInitialEventsToListener(listener, ALL_EVENTS)
-        if rc is False:
-            print("add listener for target failed")
-            return
+        if not rc:
+            print("add initial events for target failed")
+            # return
         rc = proc.GetBroadcaster().AddInitialEventsToListener(listener, ALL_EVENTS)
-        if rc is False:
-            print("add listener for process failed")
-            return
+        if not rc:
+            print("add initial events for process failed")
+            # return
 
-        rc = listener.StartListeningForEventClass(util.get_debugger(), lldb.SBThread.GetBroadcasterClassName(), ALL_EVENTS)
-        if rc is False:
+        rc = listener.StartListeningForEventClass(
+            util.get_debugger(), lldb.SBThread.GetBroadcasterClassName(), ALL_EVENTS)
+        if not rc:
             print("add listener for threads failed")
-            return
-        # THIS WILL NOT WORK: listener = util.get_debugger().GetListener()      
-        
+            # return
+        # THIS WILL NOT WORK: listener = util.get_debugger().GetListener()
+
         while True:
             event_recvd = False
-            while event_recvd is False:
+            while not event_recvd:
                 if listener.WaitForEvent(lldb.UINT32_MAX, self.event):
                     try:
-                        self.func(listener, self.event)
-                        while listener.GetNextEvent(self.event):
-                            self.func(listener, self.event)
-                        event_recvd = True
-                    except Exception as e:
+                        result = self.func(listener, self.event)
+                        if result is QUIT:
+                            return
+                    except BaseException as e:
                         print(e)
+                    while listener.GetNextEvent(self.event):
+                        try:
+                            result = self.func(listener, self.event)
+                            if result is QUIT:
+                                return
+                        except BaseException as e:
+                            print(e)
+                    event_recvd = True
             proc = util.get_process()
             if proc is not None and not proc.is_alive:
                 break
         return
 
+
 """   
     # Not sure if this is possible in LLDB...
     
@@ -385,40 +376,40 @@ class EventThread(threading.Thread):
 """
 
 
-def on_new_process(event):
+def on_new_process(event: lldb.SBEvent) -> None:
     trace = commands.STATE.trace
     if trace is None:
         return
-    with commands.STATE.client.batch():
-        with trace.open_tx("New Process {}".format(event.process.num)):
+    with trace.client.batch():
+        with trace.open_tx(f"New Process {event.process.num}"):
             commands.put_processes()  # TODO: Could put just the one....
 
 
-def on_process_selected():
+def on_process_selected() -> None:
     proc = util.get_process()
     if proc.GetProcessID() not in PROC_STATE:
         return
     trace = commands.STATE.trace
     if trace is None:
         return
-    with commands.STATE.client.batch():
-        with trace.open_tx("Process {} selected".format(proc.GetProcessID())):
+    with trace.client.batch():
+        with trace.open_tx(f"Process {proc.GetProcessID()} selected"):
             PROC_STATE[proc.GetProcessID()].record()
             commands.activate()
 
 
-def on_process_deleted(event):
+def on_process_deleted(event: lldb.SBEvent) -> None:
     trace = commands.STATE.trace
     if trace is None:
         return
     if event.process.num in PROC_STATE:
         del PROC_STATE[event.process.num]
-    with commands.STATE.client.batch():
-        with trace.open_tx("Process {} deleted".format(event.process.num)):
+    with trace.client.batch():
+        with trace.open_tx(f"Process {event.process.num} deleted"):
             commands.put_processes()  # TODO: Could just delete the one....
 
 
-def on_new_thread(event):
+def on_new_thread(event: lldb.SBEvent) -> None:
     proc = util.get_process()
     if proc.GetProcessID() not in PROC_STATE:
         return
@@ -426,264 +417,237 @@ def on_new_thread(event):
     # TODO: Syscall clone/exit to detect thread destruction?
 
 
-def on_thread_selected():
+def on_thread_selected() -> bool:
     proc = util.get_process()
     if proc.GetProcessID() not in PROC_STATE:
-        return
+        return False
     trace = commands.STATE.trace
     if trace is None:
-        return
+        return False
     t = util.selected_thread()
-    with commands.STATE.client.batch():
-        with trace.open_tx("Thread {}.{} selected".format(proc.GetProcessID(), t.GetThreadID())):
+    with trace.client.batch():
+        with trace.open_tx(f"Thread {proc.GetProcessID()}.{t.GetThreadID()} selected"):
             PROC_STATE[proc.GetProcessID()].record()
             commands.put_threads()
             commands.activate()
+    return True
 
 
-def on_frame_selected():
+def on_frame_selected() -> bool:
     proc = util.get_process()
     if proc.GetProcessID() not in PROC_STATE:
-        return
+        return False
     trace = commands.STATE.trace
     if trace is None:
-        return
+        return False
     f = util.selected_frame()
     t = f.GetThread()
-    with commands.STATE.client.batch():
-        with trace.open_tx("Frame {}.{}.{} selected".format(proc.GetProcessID(), t.GetThreadID(), f.GetFrameID())):
+    with trace.client.batch():
+        with trace.open_tx(f"Frame {proc.GetProcessID()}.{t.GetThreadID()}.{f.GetFrameID()} selected"):
             PROC_STATE[proc.GetProcessID()].record()
             commands.put_threads()
             commands.put_frames()
             commands.activate()
+    return True
 
 
-def on_syscall_memory():
+def on_syscall_memory() -> bool:
     proc = util.get_process()
     if proc.GetProcessID() not in PROC_STATE:
-        return
+        return False
     PROC_STATE[proc.GetProcessID()].regions = True
+    return True
 
 
-def on_memory_changed(event):
+def on_memory_changed(event: lldb.SBEvent) -> bool:
     proc = util.get_process()
     if proc.GetProcessID() not in PROC_STATE:
-        return
+        return False
     trace = commands.STATE.trace
     if trace is None:
-        return
-    with commands.STATE.client.batch():
-        with trace.open_tx("Memory *0x{:08x} changed".format(event.address)):
+        return False
+    with trace.client.batch():
+        with trace.open_tx(f"Memory *0x{event.address:08x} changed"):
             commands.put_bytes(event.address, event.address + event.length,
-                               pages=False, is_mi=False, from_tty=False)
+                               pages=False, result=None)
+    return True
 
 
-def on_register_changed(event):
-    print("Register changed: {}".format(dir(event)))
+def on_register_changed(event: lldb.SBEvent) -> bool:
     proc = util.get_process()
     if proc.GetProcessID() not in PROC_STATE:
-        return
+        return False
     trace = commands.STATE.trace
     if trace is None:
-        return
-    # I'd rather have a descriptor!
-    # TODO: How do I get the descriptor from the number?
-    # For now, just record the lot
-    with commands.STATE.client.batch():
-        with trace.open_tx("Register {} changed".format(event.regnum)):
+        return False
+    with trace.client.batch():
+        with trace.open_tx(f"Register {event.regnum} changed"):
             banks = event.frame.GetRegisters()
             commands.putreg(
                 event.frame, banks.GetFirstValueByName(commands.DEFAULT_REGISTER_BANK))
+    return True
 
 
-def on_cont(event):
+def on_cont(event: lldb.SBEvent) -> bool:
     proc = util.get_process()
     if proc.GetProcessID() not in PROC_STATE:
-        return
+        return False
     trace = commands.STATE.trace
     if trace is None:
-        return
+        return False
     state = PROC_STATE[proc.GetProcessID()]
-    with commands.STATE.client.batch():
+    with trace.client.batch():
         with trace.open_tx("Continued"):
             state.record_continued()
+    return True
 
 
-def on_stop(event):
-    proc = lldb.SBProcess.GetProcessFromEvent(event)
+def on_stop(event: lldb.SBEvent) -> bool:
+    proc = lldb.SBProcess.GetProcessFromEvent(
+        event) if event is not None else util.get_process()
     if proc.GetProcessID() not in PROC_STATE:
         print("not in state")
-        return
+        return False
     trace = commands.STATE.trace
     if trace is None:
         print("no trace")
-        return
+        return False
     state = PROC_STATE[proc.GetProcessID()]
     state.visited.clear()
-    with commands.STATE.client.batch():
+    with trace.client.batch():
         with trace.open_tx("Stopped"):
             state.record("Stopped")
             commands.put_event_thread()
             commands.put_threads()
             commands.put_frames()
             commands.activate()
+    return True
 
 
-def on_exited(event):
+def on_exited(event: lldb.SBEvent) -> bool:
     proc = util.get_process()
     if proc.GetProcessID() not in PROC_STATE:
-        return
+        return False
     trace = commands.STATE.trace
     if trace is None:
-        return
+        return False
     state = PROC_STATE[proc.GetProcessID()]
     state.visited.clear()
     exit_code = proc.GetExitStatus()
     description = "Exited with code {}".format(exit_code)
-    with commands.STATE.client.batch():
+    with trace.client.batch():
         with trace.open_tx(description):
             state.record(description)
             state.record_exited(exit_code)
             commands.put_event_thread()
             commands.activate()
-
-def notify_others_breaks(proc):
-    for num, state in PROC_STATE.items():
-        if num != proc.GetProcessID():
-            state.breaks = True
-
-def notify_others_watches(proc):
-    for num, state in PROC_STATE.items():
-        if num != proc.GetProcessID():
-            state.watches = True
+    return False
 
 
-def modules_changed():
+def modules_changed() -> bool:
     # Assumption: affects the current process
     proc = util.get_process()
     if proc.GetProcessID() not in PROC_STATE:
-        return
+        return False
     PROC_STATE[proc.GetProcessID()].modules = True
+    return True
 
 
-def on_new_objfile(event):
+def on_new_objfile(event: lldb.SBEvent) -> bool:
     modules_changed()
+    return True
 
 
-def on_free_objfile(event):
+def on_free_objfile(event: lldb.SBEvent) -> bool:
     modules_changed()
+    return True
 
 
-def on_breakpoint_created(b):
+def on_breakpoint_created(b: lldb.SBBreakpoint) -> bool:
     proc = util.get_process()
-    notify_others_breaks(proc)
     if proc.GetProcessID() not in PROC_STATE:
-        return
+        return False
     trace = commands.STATE.trace
     if trace is None:
-        return
-    ibpath = commands.PROC_BREAKS_PATTERN.format(procnum=proc.GetProcessID())
-    with commands.STATE.client.batch():
+        return False
+    with trace.client.batch():
         with trace.open_tx("Breakpoint {} created".format(b.GetID())):
-            ibobj = trace.create_object(ibpath)
-            # Do not use retain_values or it'll remove other locs
-            commands.put_single_breakpoint(b, ibobj, proc, [])
-            ibobj.insert()
+            commands.put_single_breakpoint(b, proc)
+    return True
 
 
-def on_breakpoint_modified(b):
+def on_breakpoint_modified(b: lldb.SBBreakpoint) -> bool:
     proc = util.get_process()
-    notify_others_breaks(proc)
     if proc.GetProcessID() not in PROC_STATE:
-        return
-    old_count = BRK_STATE.get_brkloc_count(b)
+        return False
     trace = commands.STATE.trace
     if trace is None:
-        return
-    ibpath = commands.PROC_BREAKS_PATTERN.format(procnum=proc.GetProcessID())
-    with commands.STATE.client.batch():
+        return False
+    with trace.client.batch():
         with trace.open_tx("Breakpoint {} modified".format(b.GetID())):
-            ibobj = trace.create_object(ibpath)
-            commands.put_single_breakpoint(b, ibobj, proc, [])
-            new_count = BRK_STATE.get_brkloc_count(b)
-            # NOTE: Location may not apply to process, but whatever.
-            for i in range(new_count, old_count):
-                ikey = commands.PROC_BREAK_KEY_PATTERN.format(
-                    breaknum=b.GetID(), locnum=i+1)
-                ibobj.set_value(ikey, None)
+            commands.put_single_breakpoint(b, proc)
+    return True
 
 
-def on_breakpoint_deleted(b):
+def on_breakpoint_deleted(b: lldb.SBBreakpoint) -> bool:
     proc = util.get_process()
-    notify_others_breaks(proc)
     if proc.GetProcessID() not in PROC_STATE:
-        return
-    old_count = BRK_STATE.del_brkloc_count(b.GetID())
+        return False
     trace = commands.STATE.trace
     if trace is None:
-        return
-    bpath = commands.BREAKPOINT_PATTERN.format(breaknum=b.GetID())
-    ibobj = trace.proxy_object_path(
-        commands.PROC_BREAKS_PATTERN.format(procnum=proc.GetProcessID()))
-    with commands.STATE.client.batch():
+        return False
+    bpt_path = commands.PROC_BREAK_PATTERN.format(
+        procnum=proc.GetProcessID(), breaknum=b.GetID())
+    bpt_obj = trace.proxy_object_path(bpt_path)
+    with trace.client.batch():
         with trace.open_tx("Breakpoint {} deleted".format(b.GetID())):
-            trace.proxy_object_path(bpath).remove(tree=True)
-            for i in range(old_count):
-                ikey = commands.PROC_BREAK_KEY_PATTERN.format(
-                    breaknum=b.GetID(), locnum=i+1)
-                ibobj.set_value(ikey, None)
+            bpt_obj.remove(tree=True)
+    return True
 
 
-def on_watchpoint_created(b):
+def on_watchpoint_created(b: lldb.SBWatchpoint) -> bool:
     proc = util.get_process()
-    notify_others_watches(proc)
     if proc.GetProcessID() not in PROC_STATE:
-        return
+        return False
     trace = commands.STATE.trace
     if trace is None:
-        return
-    ibpath = commands.PROC_WATCHES_PATTERN.format(procnum=proc.GetProcessID())
-    with commands.STATE.client.batch():
+        return False
+    with trace.client.batch():
         with trace.open_tx("Breakpoint {} created".format(b.GetID())):
-            ibobj = trace.create_object(ibpath)
-            # Do not use retain_values or it'll remove other locs
-            commands.put_single_watchpoint(b, ibobj, proc, [])
-            ibobj.insert()
+            commands.put_single_watchpoint(b, proc)
+    return True
 
 
-def on_watchpoint_modified(b):
+def on_watchpoint_modified(b: lldb.SBWatchpoint) -> bool:
     proc = util.get_process()
-    notify_others_watches(proc)
     if proc.GetProcessID() not in PROC_STATE:
-        return
-    old_count = BRK_STATE.get_brkloc_count(b)
+        return False
     trace = commands.STATE.trace
     if trace is None:
-        return
-    ibpath = commands.PROC_WATCHES_PATTERN.format(procnum=proc.GetProcessID())
-    with commands.STATE.client.batch():
+        return False
+    with trace.client.batch():
         with trace.open_tx("Watchpoint {} modified".format(b.GetID())):
-            ibobj = trace.create_object(ibpath)
-            commands.put_single_watchpoint(b, ibobj, proc, [])
+            commands.put_single_watchpoint(b, proc)
+    return True
 
 
-def on_watchpoint_deleted(b):
+def on_watchpoint_deleted(b: lldb.SBWatchpoint) -> bool:
     proc = util.get_process()
-    notify_others_watches(proc)
     if proc.GetProcessID() not in PROC_STATE:
-        return
+        return False
     trace = commands.STATE.trace
     if trace is None:
-        return
-    bpath = commands.WATCHPOINT_PATTERN.format(watchnum=b.GetID())
-    ibobj = trace.proxy_object_path(
-        commands.PROC_WATCHES_PATTERN.format(procnum=proc.GetProcessID()))
-    with commands.STATE.client.batch():
+        return False
+    wpt_path = commands.PROC_WATCH_PATTERN.format(
+        procnum=proc.GetProcessID(), watchnum=b.GetID())
+    wpt_obj = trace.proxy_object_path(wpt_path)
+    with trace.client.batch():
         with trace.open_tx("Watchpoint {} deleted".format(b.GetID())):
-            trace.proxy_object_path(bpath).remove(tree=True)
+            wpt_obj.remove(tree=True)
+    return True
 
 
-def install_hooks():
+def install_hooks() -> None:
     if HOOK_STATE.installed:
         return
     HOOK_STATE.installed = True
@@ -692,17 +656,18 @@ def install_hooks():
     event_thread.start()
 
 
-def remove_hooks():
+def remove_hooks() -> None:
     if not HOOK_STATE.installed:
         return
     HOOK_STATE.installed = False
 
-def enable_current_process():
+
+def enable_current_process() -> None:
     proc = util.get_process()
     PROC_STATE[proc.GetProcessID()] = ProcessState()
 
 
-def disable_current_process():
+def disable_current_process() -> None:
     proc = util.get_process()
     if proc.GetProcessID() in PROC_STATE:
         # Silently ignore already disabled
