@@ -4,9 +4,9 @@
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
  * You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -31,6 +31,8 @@ ElementId ELEM_NORMADDR = ElementId("normaddr",215);
 ElementId ELEM_NORMHASH = ElementId("normhash",216);
 ElementId ELEM_STARTVAL = ElementId("startval",217);
 
+const uint8 JumpValues::NO_LABEL = 0xBAD1ABE1BAD1ABE1;
+
 /// \param encoder is the stream encoder
 void LoadTable::encode(Encoder &encoder) const
 
@@ -1047,7 +1049,7 @@ void JumpBasic::analyzeGuards(BlockBasic *bl,int4 pathout)
   int4 i,j,indpath;
   int4 maxbranch = 2;		// Maximum number of CBRANCHs to consider
   int4 maxpullback = 2;
-  bool usenzmask = (jumptable->getStage() == 0);
+  bool usenzmask = !jumptable->isPartial();
 
   selectguards.clear();
   BlockBasic *prevbl;
@@ -1282,6 +1284,18 @@ bool JumpBasic::flowsOnlyToModel(Varnode *vn,PcodeOp *trailOp)
   return true;
 }
 
+/// \param arr is the array of Varnodes
+/// \return \b true if all elements are the same
+bool JumpBasic::duplicateVarnodes(const vector<Varnode *> &arr)
+
+{
+  Varnode *vn = arr[0];
+  for(int4 i=1;i<arr.size();++i) {
+    if (arr[i] != vn) return false;
+  }
+  return true;
+}
+
 /// All CBRANCHs in addition to flowing to the given block, must also flow to another common block,
 /// and each boolean value must select between the given block and the common block in the same way.
 /// If this flow exists, \b true is returned and the boolean Varnode inputs to each CBRANCH are passed back.
@@ -1336,9 +1350,14 @@ void JumpBasic::checkUnrolledGuard(BlockBasic *bl,int4 maxpullback,bool usenzmas
   int4 indpathstore = bl->getIn(0)->getFlipPath() ? 1-indpath : indpath;
   PcodeOp *readOp = cbranch;
   for(int4 j=0;j<maxpullback;++j) {
-    PcodeOp *multiOp = bl->findMultiequal(varArray);
-    if (multiOp != (PcodeOp *)0) {
-      selectguards.push_back(GuardRecord(cbranch,readOp,indpathstore,rng,multiOp->getOut(),true));
+    if (duplicateVarnodes(varArray)) {
+      selectguards.push_back(GuardRecord(cbranch,readOp,indpathstore,rng,varArray[0],true));
+    }
+    else {
+      PcodeOp *multiOp = bl->findMultiequal(varArray);
+      if (multiOp != (PcodeOp *)0) {
+	selectguards.push_back(GuardRecord(cbranch,readOp,indpathstore,rng,multiOp->getOut(),true));
+      }
     }
     Varnode *markup;		// Throw away markup information
     Varnode *vn = varArray[0];
@@ -1355,42 +1374,38 @@ bool JumpBasic::foldInOneGuard(Funcdata *fd,GuardRecord &guard,JumpTable *jump)
 
 {
   PcodeOp *cbranch = guard.getBranch();
-  int4 indpath = guard.getPath();	// Get stored path to indirect block
   BlockBasic *cbranchblock = cbranch->getParent();
-  if (cbranchblock->getFlipPath()) // Based on whether out branches have been flipped
-    indpath = 1 - indpath;	// get actual path to indirect block
-  BlockBasic *guardtarget = (BlockBasic *)cbranchblock->getOut(1-indpath);
-  bool change = false;
-  int4 pos;
-
   // Its possible the guard branch has been converted between the switch recovery and now
   if (cbranchblock->sizeOut() != 2) return false; // In which case, we can't fold it in
+  int4 indpath = guard.getPath();	// Get stored path to indirect block
+  if (cbranchblock->getFlipPath()) // Based on whether out branches have been flipped
+    indpath = 1 - indpath;	// get actual path to indirect block
   BlockBasic *switchbl = jump->getIndirectOp()->getParent();
+  if (cbranchblock->getOut(indpath) != switchbl) // Guard must go directly into switch block
+    return false;
+  BlockBasic *guardtarget = (BlockBasic *)cbranchblock->getOut(1-indpath);
+  int4 pos;
+
   for(pos=0;pos<switchbl->sizeOut();++pos)
     if (switchbl->getOut(pos) == guardtarget) break;
+  if (jump->hasFoldedDefault() && jump->getDefaultBlock() != pos)	// There can be only one folded target
+    return false;
+
+  if (!switchbl->noInterveningStatement())
+    return false;
   if (pos == switchbl->sizeOut()) {
-    if (BlockBasic::noInterveningStatement(cbranch,indpath,switchbl->lastOp())) {
-      // Adjust tables and control flow graph
-      // for new jumptable destination
-      jump->addBlockToSwitch(guardtarget,0xBAD1ABE1);
-      jump->setLastAsMostCommon();
-      fd->pushBranch(cbranchblock,1-indpath,switchbl);
-      guard.clear();
-      change = true;
-    }
+    jump->addBlockToSwitch(guardtarget,JumpValues::NO_LABEL);	// Add new destination to table without a label
+    jump->setLastAsDefault();			// treating it as either the default case or an exit
+    fd->pushBranch(cbranchblock,1-indpath,switchbl);	// Turn branch target into target of the switch instead
   }
   else {
-    // We should probably check that there are no intervening
-    // statements between the guard and the switch. But the
-    // fact that the guard target is also a switch target
-    // is a good indicator that there are none
     uintb val = ((indpath==0)!=(cbranch->isBooleanFlip())) ? 0 : 1;
     fd->opSetInput(cbranch,fd->newConstant(cbranch->getIn(0)->getSize(),val),1);
     jump->setDefaultBlock(pos);	// A guard branch generally targets the default case
-    guard.clear();
-    change = true;
   }
-  return change;
+  jump->setFoldedDefault();	// Mark that the default branch has been folded (and cannot take a label)
+  guard.clear();
+  return true;
 }
 
 JumpBasic::~JumpBasic(void)
@@ -1504,12 +1519,12 @@ void JumpBasic::buildLabels(Funcdata *fd,vector<Address> &addresstable,vector<ui
       try {
 	switchval = backup2Switch(fd,val,normalvn,switchvn);		// Do reverse emulation to get original switch value
       } catch(EvaluationError &err) {
-	switchval = 0xBAD1ABE1;
+	switchval = JumpValues::NO_LABEL;
 	needswarning = 2;
       }
     }
     else
-      switchval = 0xBAD1ABE1;	// If can't reverse, hopefully this is the default or exit, otherwise give "badlabel"
+      switchval = JumpValues::NO_LABEL;	// If can't reverse, hopefully this is the default or exit
     if (needswarning==1)
       fd->warning("This code block may not be properly labeled as switch case",addresstable[label.size()]);
     else if (needswarning==2)
@@ -1524,7 +1539,7 @@ void JumpBasic::buildLabels(Funcdata *fd,vector<Address> &addresstable,vector<ui
 
   while(label.size() < addresstable.size()) {
     fd->warning("Bad switch case",addresstable[label.size()]);
-    label.push_back(0xBAD1ABE1);
+    label.push_back(JumpValues::NO_LABEL);
   }
 }
 
@@ -1628,7 +1643,7 @@ bool JumpBasic2::foldInOneGuard(Funcdata *fd,GuardRecord &guard,JumpTable *jump)
   // So we don't make any special mods, in case there are extra statements in these blocks
 
   // The final block in the table is the single value produced by the model2 guard
-  jump->setLastAsMostCommon();	// It should be the default block
+  jump->setLastAsDefault();	// It should be the default block
   guard.clear();		// Mark that we are folded
   return true;
 }
@@ -1977,7 +1992,7 @@ void JumpBasicOverride::buildLabels(Funcdata *fd,vector<Address> &addresstable,v
     try {
       addr = backup2Switch(fd,values[i],normalvn,switchvn);
     } catch(EvaluationError &err) {
-      addr = 0xBAD1ABE1;
+      addr = JumpValues::NO_LABEL;
     }
     label.push_back(addr);
     if (label.size() >= addresstable.size()) break; // This should never happen
@@ -1985,7 +2000,7 @@ void JumpBasicOverride::buildLabels(Funcdata *fd,vector<Address> &addresstable,v
 
   while(label.size() < addresstable.size()) {
     fd->warning("Bad switch case",addresstable[label.size()]); // This should never happen
-    label.push_back(0xBAD1ABE1);
+    label.push_back(JumpValues::NO_LABEL);
   }
 }
 
@@ -2084,8 +2099,10 @@ bool JumpAssisted::recoverModel(Funcdata *fd,PcodeOp *indop,uint4 matchsize,uint
   if (assistOp->code() != CPUI_CALLOTHER) return false;
   if (assistOp->numInput() < 3) return false;
   int4 index = assistOp->getIn(0)->getOffset();
-  userop = dynamic_cast<JumpAssistOp *>(fd->getArch()->userops.getOp(index));
-  if (userop == (JumpAssistOp *)0) return false;
+  UserPcodeOp *tmpOp = fd->getArch()->userops.getOp(index);
+  if (tmpOp->getType() != UserPcodeOp::jumpassist)
+    return false;
+  userop = (JumpAssistOp *)tmpOp;
 
   switchvn = assistOp->getIn(1);		// The switch variable
   for(int4 i=2;i<assistOp->numInput();++i)
@@ -2170,7 +2187,7 @@ void JumpAssisted::buildLabels(Funcdata *fd,vector<Address> &addresstable,vector
       label.push_back(output);
     }
   }
-  label.push_back(0xBAD1ABE1);		// Add fake label to match the defaultAddress
+  label.push_back(JumpValues::NO_LABEL);	// Add fake label to match the defaultAddress
 }
 
 Varnode *JumpAssisted::foldInNormalization(Funcdata *fd,PcodeOp *indop)
@@ -2192,7 +2209,7 @@ bool JumpAssisted::foldInGuards(Funcdata *fd,JumpTable *jump)
 
 {
   int4 origVal = jump->getDefaultBlock();
-  jump->setLastAsMostCommon();			// Default case is always the last block
+  jump->setLastAsDefault();			// Default case is always the last block
   return (origVal != jump->getDefaultBlock());
 }
 
@@ -2205,6 +2222,33 @@ JumpModel *JumpAssisted::clone(JumpTable *jt) const
   return clone;
 }
 
+void JumpTable::saveModel(void)
+
+{
+  if (origmodel != (JumpModel *)0)
+    delete origmodel;
+  origmodel = jmodel;
+  jmodel = (JumpModel *)0;
+}
+
+void JumpTable::restoreSavedModel(void)
+
+{
+  if (jmodel != (JumpModel *)0)
+    delete jmodel;
+  jmodel = origmodel;
+  origmodel = (JumpModel *)0;
+}
+
+void JumpTable::clearSavedModel(void)
+
+{
+  if (origmodel != (JumpModel *)0) {
+    delete origmodel;
+    origmodel = (JumpModel *)0;
+  }
+}
+
 /// Try to recover each model in turn, until we find one that matches the specific BRANCHIND.
 /// \param fd is the function containing the switch
 void JumpTable::recoverModel(Funcdata *fd)
@@ -2252,12 +2296,12 @@ void JumpTable::sanityCheck(Funcdata *fd,vector<int4> *loadcounts)
 
 {
   if (jmodel->isOverride())
-    return;			// Don't perform sanity check on an override
+    return;				// Don't perform sanity check on an override
   uint4 sz = addresstable.size();
 
   if (!isReachable(indirect))
-    throw JumptableNotReachableError("No legal flow");
-  if (addresstable.size() == 1) { // One entry is likely some kind of thunk
+    partialTable = true;		// If the jumptable is not reachable, mark as incomplete
+  if (addresstable.size() == 1) { 	// One entry is likely some kind of thunk
     bool isthunk = false;
     uintb diff;
     Address addr = addresstable[0];
@@ -2346,8 +2390,9 @@ JumpTable::JumpTable(Architecture *g,Address ad)
   maxaddsub = 1;
   maxleftright = 1;
   maxext = 1;
-  recoverystage = 0;
+  partialTable = false;
   collectloads = false;
+  defaultIsFolded = false;
 }
 
 /// This is a partial clone of another jump-table. Objects that are specific
@@ -2366,8 +2411,9 @@ JumpTable::JumpTable(const JumpTable *op2)
   maxaddsub = op2->maxaddsub;
   maxleftright = op2->maxleftright;
   maxext = op2->maxext;
-  recoverystage = op2->recoverystage;
+  partialTable = op2->partialTable;
   collectloads = op2->collectloads;
+  defaultIsFolded = false;
 				// We just clone the addresses themselves
   addresstable = op2->addresstable;
   loadpoints = op2->loadpoints;
@@ -2453,7 +2499,7 @@ int4 JumpTable::getIndexByBlock(const FlowBlock *bl,int4 i) const
   throw LowlevelError("Could not get jumptable index for block");
 }
 
-void JumpTable::setLastAsMostCommon(void)
+void JumpTable::setLastAsDefault(void)
 
 {
   defaultBlock = lastBlock;
@@ -2585,8 +2631,8 @@ void JumpTable::recoverAddresses(Funcdata *fd)
   }
   if (jmodel->getTableSize() == 0) {
     ostringstream err;
-    err << "Impossible to reach jumptable at " << opaddress;
-    throw JumptableNotReachableError(err.str());
+    err << "Jumptable with 0 entries at " << opaddress;
+    throw LowlevelError(err.str());
   }
   //  if (sz < 2)
   //    fd->warning("Jumptable has only one branch",opaddress);
@@ -2607,10 +2653,7 @@ void JumpTable::recoverAddresses(Funcdata *fd)
 void JumpTable::recoverMultistage(Funcdata *fd)
 
 {
-  if (origmodel != (JumpModel *)0)
-    delete origmodel;
-  origmodel = jmodel;
-  jmodel = (JumpModel *)0;
+  saveModel();
   
   vector<Address> oldaddresstable = addresstable;
   addresstable.clear();
@@ -2619,36 +2662,25 @@ void JumpTable::recoverMultistage(Funcdata *fd)
     recoverAddresses(fd);
   }
   catch(JumptableThunkError &err) {
-    if (jmodel != (JumpModel *)0)
-      delete jmodel;
-    jmodel = origmodel;
-    origmodel = (JumpModel *)0;
+    restoreSavedModel();
     addresstable = oldaddresstable;
     fd->warning("Second-stage recovery error",indirect->getAddr());
   }
   catch(LowlevelError &err) {
-    if (jmodel != (JumpModel *)0)
-      delete jmodel;
-    jmodel = origmodel;
-    origmodel = (JumpModel *)0;
+    restoreSavedModel();
     addresstable = oldaddresstable;
     fd->warning("Second-stage recovery error",indirect->getAddr());
   }
-  recoverystage = 2;
-  if (origmodel != (JumpModel *)0) { // Keep the new model if it was created successfully
-    delete origmodel;
-    origmodel = (JumpModel *)0;
-  }
+  partialTable = false;
+  clearSavedModel();		// Keep the new model if it was created successfully
 }
 
-/// This is run assuming the address table has already been recovered, via recoverAddresses() in another
-/// Funcdata instance. So recoverModel() needs to be rerun on the instance passed in here.
-///
-/// The unnormalized switch variable is recovered, and for each possible address table entry, the variable
-/// value that produces it is calculated and stored as the formal \e case label for the associated code block.
-/// \param fd is the (final instance of the) function containing the switch
-/// \return \b true if it looks like a multi-stage restart is needed.
-bool JumpTable::recoverLabels(Funcdata *fd)
+/// This assumes the address table has already been recovered, via recoverAddresses() in another
+/// Funcdata instance. We rerun recoverModel() to match with the current Funcdata.  If the recovered model
+/// does not match the original address table size, we may be missing control-flow. In this case,
+/// if it looks like we have a \e multistage jumptable, we generate a multistage restart, otherwise
+/// we generate a warning of the mismatch.
+void JumpTable::matchModel(Funcdata *fd)
 
 {
   if (!isRecovered())
@@ -2656,24 +2688,33 @@ bool JumpTable::recoverLabels(Funcdata *fd)
 
   // Unless the model is an override, move model (created on a flow copy) so we can create a current instance
   if (jmodel != (JumpModel *)0) {
-    if (origmodel != (JumpModel *)0)
-      delete origmodel;
-    if (!jmodel->isOverride()) {
-      origmodel = jmodel;
-      jmodel = (JumpModel *)0;
-    }
-    else
+    if (!jmodel->isOverride())
+      saveModel();
+    else {
+      clearSavedModel();
       fd->warning("Switch is manually overridden",opaddress);
-  }
-
-  bool multistagerestart = false;
-  recoverModel(fd);		// Create a current instance of the model
-  if (jmodel != (JumpModel *)0) {
-    if (jmodel->getTableSize() != addresstable.size()) {
-      fd->warning("Could not find normalized switch variable to match jumptable",opaddress);
-      if ((addresstable.size()==1)&&(jmodel->getTableSize() > 1))
-	multistagerestart = true;
     }
+  }
+  recoverModel(fd);		// Create a current instance of the model
+  if (jmodel != (JumpModel *)0 && jmodel->getTableSize() != addresstable.size()) {
+    if ((addresstable.size()==1)&&(jmodel->getTableSize() > 1)) {
+      // The jumptable was not fully recovered during flow analysis, try to issue a restart
+      fd->getOverride().insertMultistageJump(opaddress);
+      fd->setRestartPending(true);
+      return;
+    }
+    fd->warning("Could not find normalized switch variable to match jumptable",opaddress);
+  }
+}
+
+
+/// The unnormalized switch variable is recovered, and for each possible address table entry, the variable
+/// value that produces it is calculated and stored as the formal \e case label for the associated code block.
+/// \param fd is the (final instance of the) function containing the switch
+void JumpTable::recoverLabels(Funcdata *fd)
+
+{
+  if (jmodel != (JumpModel *)0) {
     if ((origmodel == (JumpModel *)0)||(origmodel->getTableSize()==0)) {
       jmodel->findUnnormalized(maxaddsub,maxleftright,maxext);
       jmodel->buildLabels(fd,addresstable,label,jmodel);
@@ -2690,11 +2731,7 @@ bool JumpTable::recoverLabels(Funcdata *fd)
     trivialSwitchOver();
     jmodel->buildLabels(fd,addresstable,label,origmodel);
   }
-  if (origmodel != (JumpModel *)0) {
-    delete origmodel;
-    origmodel = (JumpModel *)0;
-  }
-  return multistagerestart;
+  clearSavedModel();
 }
 
 /// Clear out any data that is specific to a Funcdata instance.
@@ -2702,10 +2739,7 @@ bool JumpTable::recoverLabels(Funcdata *fd)
 void JumpTable::clear(void)
 
 {
-  if (origmodel != (JumpModel *)0) {
-    delete origmodel;
-    origmodel = (JumpModel *)0;
-  }
+  clearSavedModel();
   if (jmodel->isOverride())
     jmodel->clear();
   else {
@@ -2720,7 +2754,7 @@ void JumpTable::clear(void)
   indirect = (PcodeOp *)0;
   switchVarConsume = ~((uintb)0);
   defaultBlock = -1;
-  recoverystage = 0;
+  partialTable = false;
   // -opaddress- -maxtablesize- -maxaddsub- -maxleftright- -maxext- -collectloads- are permanent
 }
 
@@ -2742,7 +2776,7 @@ void JumpTable::encode(Encoder &encoder) const
     if (spc != (AddrSpace *)0)
       spc->encodeAttributes(encoder,off);
     if (i<label.size()) {
-      if (label[i] != 0xBAD1ABE1)
+      if (label[i] != JumpValues::NO_LABEL)
 	encoder.writeUnsignedInteger(ATTRIB_LABEL, label[i]);
     }
     encoder.closeElement(ELEM_DEST);
@@ -2802,7 +2836,7 @@ void JumpTable::decode(Decoder &decoder)
 
   if (label.size()!=0) {
     while(label.size() < addresstable.size())
-      label.push_back(0xBAD1ABE1);
+      label.push_back(JumpValues::NO_LABEL);
   }
 }
 
@@ -2814,11 +2848,11 @@ bool JumpTable::checkForMultistage(Funcdata *fd)
 
 {
   if (addresstable.size()!=1) return false;
-  if (recoverystage != 0) return false;
+  if (partialTable) return false;
   if (indirect == (PcodeOp *)0) return false;
 
   if (fd->getOverride().queryMultistageJumptable(indirect->getAddr())) {
-    recoverystage = 1;		// Mark that we need additional recovery
+    partialTable = true;		// Mark that we need additional recovery
     return true;
   }
   return false;
