@@ -4,9 +4,9 @@
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
  * You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -361,25 +361,29 @@ class ProgramDBChangeSet implements ProgramChangeSet, DomainObjectDBChangeSet {
 		}
 
 		if (!isCheckedOut) { // if not versioned, wipe out change sets
-			changedAddrsSinceCheckout.clear();
-			changedRegAddrsSinceCheckout.clear();
-			changedAddrsSinceSave.clear();
-			changedRegAddrsSinceSave.clear();
-			changedCategoryIds.clear();
-			changedDataTypeIds.clear();
-			changedProgramTreeIds.clear();
-			changedSymbolIds.clear();
-			changedSourceArchiveIds.clear();
-			addedCategoryIds.clear();
-			addedDataTypeIds.clear();
-			addedProgramTreeIds.clear();
-			addedSymbolIds.clear();
-			addedSourceArchiveIds.clear();
+			clearAll();
 		}
 
 		clearUndo();
 	}
 
+	void clearAll() {
+		changedAddrsSinceCheckout.clear();
+		changedRegAddrsSinceCheckout.clear();
+		changedAddrsSinceSave.clear();
+		changedRegAddrsSinceSave.clear();
+		changedCategoryIds.clear();
+		changedDataTypeIds.clear();
+		changedProgramTreeIds.clear();
+		changedSymbolIds.clear();
+		changedSourceArchiveIds.clear();
+		addedCategoryIds.clear();
+		addedDataTypeIds.clear();
+		addedProgramTreeIds.clear();
+		addedSymbolIds.clear();
+		addedSourceArchiveIds.clear();
+	}
+
 	@Override
 	public synchronized void startTransaction() {
 		inTransaction = true;
@@ -659,6 +663,7 @@ class ProgramDBChangeSet implements ProgramChangeSet, DomainObjectDBChangeSet {
 		}
 		return true;
 	}
+
 }
 
 class ChangeDiff {
