@@ -4,9 +4,9 @@
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
  * You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -18,6 +18,7 @@ package ghidra.program.model.data;
 import java.util.*;
 
 import db.Transaction;
+import ghidra.framework.model.DomainObject;
 import ghidra.program.database.SpecExtension;
 import ghidra.program.database.map.AddressMap;
 import ghidra.program.model.lang.*;
@@ -27,6 +28,8 @@ import ghidra.util.InvalidNameException;
 import ghidra.util.UniversalID;
 import ghidra.util.exception.CancelledException;
 import ghidra.util.task.TaskMonitor;
+import utility.function.ExceptionalCallback;
+import utility.function.ExceptionalSupplier;
 
 /**
  * Interface for Managing data types.
@@ -88,7 +91,9 @@ public interface DataTypeManager {
 
 	/**
 	 * Returns a unique name not currently used by any other dataType or category
-	 * with the same baseName
+	 * with the same baseName.  This does not produce a conflict name and is intended 
+	 * to be used when generating an artifical datatype name only (e.g., {@code temp_1},
+	 * {@code temp_2}; for {@code baseName="temp"}.
 	 *
 	 * @param path the path of the name
 	 * @param baseName the base name to be made unique
@@ -167,15 +172,19 @@ public interface DataTypeManager {
 	/**
 	 * Begin searching at the root category for all data types with the
 	 * given name. Places all the data types in this data type manager
-	 * with the given name into the list.
-	 * @param name name of the data type
+	 * with the given name into the list.  Presence of {@code .conflict}
+	 * extension will be ignored for both specified name and returned
+	 * results.
+	 * @param name name of the data type (wildcards are not supported and will be treated
+	 * as explicit search characters)
 	 * @param list list that will be populated with matching DataType objects
 	 */
 	public void findDataTypes(String name, List<DataType> list);
 
 	/**
 	 * Begin searching at the root category for all data types with names
-	 * that match the given name that may contain wildcards.
+	 * that match the given name that may contain wildcards using familiar globbing 
+	 * characters '*' and '?'.
 	 * @param name name to match; may contain wildcards
 	 * @param list list that will be populated with matching DataType objects
 	 * @param caseSensitive true if the match is case sensitive
@@ -300,7 +309,14 @@ public interface DataTypeManager {
 	public void removeInvalidatedListener(InvalidatedListener listener);
 
 	/**
-	 * Remove the given datatype from this manager
+	 * Remove the given datatype from this manager.
+	 * <br>
+	 * NOTE: Any use of the specified datatype within a {@link FunctionDefinition} will be 
+	 * converted to the {@link DataType#DEFAULT default 'undefined' datatype}.  Any use within
+	 * a {@link Structure} or {@link Union} will be converted to the {@link BadDataType} as
+	 * a placeholder to retain the component's field name and length (the comment will be prefixed
+	 * with a message indicating the remval of the old datatype.
+	 * 
 	 * @param dataType the dataType to be removed
 	 * @param monitor the task monitor
 	 * @return true if the data type existed and was removed
@@ -372,11 +388,91 @@ public interface DataTypeManager {
 	public int startTransaction(String description);
 
 	/**
-	 * Ends the current transaction
+	 * Ends the current transaction.
+	 * <P>
+	 * NOTE: If multiple transactions are outstanding the full transaction will not be ended
+	 * until all transactions have been ended.  If any of the transactions indicate a 
+	 * false for {@code commit} the transaction will ultimately be rolled-back when the final
+	 * transaction is ended.
+	 * <P>
+	 * NOTE: Use of rollback ({@code commit=false} should be avoided unless absolutely
+	 * neccessary since it will incur overhead to revert changes and may rollback multiple
+	 * concurrent transactions if they exist.
+	 * <P>
+	 * NOTE: If this manager is part of a larger {@link DomainObject} its transactions may become
+	 * entangled with other transactions at a higher level.  In such cases, use of  the 
+	 * {@link DomainObject} transaction interface is preferred.  The return value from this
+	 * method cannot be relied on in such cases. 
 	 * @param transactionID id of the transaction to end
-	 * @param commit true if changes are committed, false if changes in transaction are revoked
+	 * @param commit true if changes are committed, false if changes in transaction should be
+	 * rolled back.
+	 * @return true if this invocation was the final transaction and all changes were comitted.
 	 */
-	public void endTransaction(int transactionID, boolean commit);
+	public boolean endTransaction(int transactionID, boolean commit);
+
+	/**
+	 * Performs the given callback inside of a transaction.  Use this method in place of the more
+	 * verbose try/catch/finally semantics.
+	 * <pre>
+	 * program.withTransaction("My Description", () -> {
+	 * 	// ... Do something
+	 * });
+	 * </pre>
+	 * 
+	 * <p>
+	 * Note: the transaction created by this method will always be committed when the call is 
+	 * finished.  If you need the ability to abort transactions, then you need to use the other 
+	 * methods on this interface.
+	 * 
+	 * @param description brief description of transaction
+	 * @param callback the callback that will be called inside of a transaction
+	 * @throws E any exception that may be thrown in the given callback
+	 */
+	public default <E extends Exception> void withTransaction(String description,
+			ExceptionalCallback<E> callback) throws E {
+		int id = startTransaction(description);
+		try {
+			callback.call();
+		}
+		finally {
+			endTransaction(id, true);
+		}
+	}
+
+	/**
+	 * Calls the given supplier inside of a transaction.  Use this method in place of the more
+	 * verbose try/catch/finally semantics.
+	 * <pre>
+	 * program.withTransaction("My Description", () -> {
+	 * 	// ... Do something
+	 * 	return result;
+	 * });
+	 * </pre>
+	 * <p>
+	 * If you do not need to supply a result, then use 
+	 * {@link #withTransaction(String, ExceptionalCallback)} instead.
+	 * 
+	 * @param <E> the exception that may be thrown from this method 
+	 * @param <T> the type of result returned by the supplier
+	 * @param description brief description of transaction
+	 * @param supplier the supplier that will be called inside of a transaction
+	 * @return the result returned by the supplier
+	 * @throws E any exception that may be thrown in the given callback
+	 */
+	public default <E extends Exception, T> T withTransaction(String description,
+			ExceptionalSupplier<T, E> supplier) throws E {
+		T t = null;
+		boolean success = false;
+		int id = startTransaction(description);
+		try {
+			t = supplier.get();
+			success = true;
+		}
+		finally {
+			endTransaction(id, success);
+		}
+		return t;
+	}
 
 	/**
 	 * Force all pending notification events to be flushed
